package com.tzld.piaoquan.longarticle.job;

import com.tzld.piaoquan.longarticle.service.local.CoreService;
import com.tzld.piaoquan.longarticle.utils.LarkRobotUtil;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import static com.tzld.piaoquan.longarticle.common.constants.RedisConstant.REDIS_COOKIE_KEY;

@Slf4j
@Component
public class PlanAccountJob {

    @Autowired
    private CoreService coreService;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @XxlJob("initAccountSpecialSettingsJob")
    public ReturnT<String> initAccountSpecialSettings(String param) {
        try {
            coreService.initAccountSpecialSettings();
        } catch (Exception e) {
            LarkRobotUtil.sendMessage("例外设置异常，请及时查看，@薛一鸣");
            log.error("initPlanAccountJob error", e);
        }
        return ReturnT.SUCCESS;
    }


    @XxlJob("initPlanAccountJob")
    public ReturnT<String> initPlanAccount(String param) {
        try {
            coreService.initPlanAccount();
        } catch (Exception e) {
            LarkRobotUtil.sendMessage("长文系统初始化异常，请及时查看，@薛一鸣");
            log.error("initPlanAccountJob error", e);
        }
        return ReturnT.SUCCESS;
    }

    @XxlJob("matchPlanAccountJob")
    public ReturnT<String> matchPlanAccount(String param) {
        try {
            coreService.matchContent();
        } catch (Exception e) {
            LarkRobotUtil.sendMessage("长文系统匹配异常，请及时查看，@薛一鸣");
            log.error("matchPlanAccountJob error", e);
        }
        return ReturnT.SUCCESS;
    }

    @XxlJob("corePlanAccountJob")
    public ReturnT<String> corePlanAccount(String param) {
        try {
            coreService.core();
        } catch (Exception e) {
            LarkRobotUtil.sendMessage("长文系统核心流程异常，请及时查看，@薛一鸣");
            log.error("corePlanAccountJob error", e);
        }
        return ReturnT.SUCCESS;
    }

    @XxlJob("getPushStatusJob")
    public ReturnT<String> getPushStatus(String param) {
        try {
            coreService.getPushStatus();
        } catch (Exception e) {
            LarkRobotUtil.sendMessage("长文系统获取结果异常，请及时查看，@薛一鸣");
            log.error("getPushStatusJob error", e);
        }
        return ReturnT.SUCCESS;
    }

    @XxlJob("updateCookieJob")
    public ReturnT<String> updateCookie(String param) {
        redisTemplate.opsForValue().set(REDIS_COOKIE_KEY, param);
        return ReturnT.SUCCESS;
    }

}
