package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.longarticle.common.enums.PublishGzhPushTypeEnum;
import com.tzld.piaoquan.longarticle.dao.mapper.CrawlerVideoMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.MatchVideoMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.PublishMatchContentMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.SingleVideoSourceMapper;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.dto.ArticleSortRequest;
import com.tzld.piaoquan.longarticle.model.dto.ArticleSortResponse;
import com.tzld.piaoquan.longarticle.model.dto.ArticleSortResponseData;
import com.tzld.piaoquan.longarticle.model.dto.PublishArticleData;
import com.tzld.piaoquan.longarticle.model.po.*;
import com.tzld.piaoquan.longarticle.model.vo.*;
import com.tzld.piaoquan.longarticle.service.local.ContentService;
import com.tzld.piaoquan.longarticle.service.local.KimiService;
import com.tzld.piaoquan.longarticle.service.remote.AigcService;
import com.tzld.piaoquan.longarticle.service.remote.SortService;
import com.tzld.piaoquan.longarticle.service.remote.VideoService;
import com.tzld.piaoquan.longarticle.utils.LarkRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

@Slf4j
@Service
public class ContentServiceImpl implements ContentService {
    @ApolloJsonValue("${testAccountLevel2:{}}")
    private Map<String, Integer> ghIdMap;

    private final String SINGLE_VIDEO_UID = "76862180";

    @Autowired
    private MatchVideoMapper matchVideoMapper;

    @Autowired
    private CrawlerVideoMapper crawlerVideoMapper;

    @Autowired
    private PublicContentServiceImpl publicContentService;

    @Autowired
    private AigcService aigcService;

    @Autowired
    private SortService sortService;

    @Autowired
    private KimiService kimiService;

    @Autowired
    private VideoService videoService;

    @Autowired
    private SingleVideoSourceMapper singleVideoSourceMapper;

    @Autowired
    private PublishMatchContentMapper publishMatchContentMapper;

    public void addMatchContent(MatchVideo matchVideo, String publishContentId) {
        PublishMatchContent publishMatchContent = new PublishMatchContent();
        publishMatchContent.setTraceId(matchVideo.getTraceId());
        publishMatchContent.setPublishContentId(publishContentId);
        publishMatchContent.setContentId(matchVideo.getContentId());
        publishMatchContent.setGhId(matchVideo.getGhId());
        publishMatchContent.setStatus(matchVideo.getContentStatus());
        publishMatchContentMapper.insertSelective(publishMatchContent);
    }

    public MatchVideo getContent(String contentId, String ghId, Integer publishFlag) {
        MatchVideoExample matchVideoExample = new MatchVideoExample();
        matchVideoExample.createCriteria().andGhIdEqualTo(ghId).andContentIdEqualTo(contentId)
                .andPublishFlagEqualTo(publishFlag);
        List<MatchVideo> matchVideos = matchVideoMapper.selectByExample(matchVideoExample);
        if (CollectionUtils.isEmpty(matchVideos)) {
            return null;
        }
        return matchVideos.get(0);
    }

    public MatchVideo getMatchVideo(PublishContent publishContent, PlanAccount planAccount) {
        MatchVideoExample matchVideoExample = new MatchVideoExample();
        matchVideoExample.createCriteria().andGhIdEqualTo(planAccount.getGhId()).andContentIdEqualTo(publishContent.getSourceId())
                .andPublishFlagIn(Arrays.asList(1, 2));
        matchVideoExample.setOrderByClause("publish_flag desc");
        List<MatchVideo> matchVideos = matchVideoMapper.selectByExample(matchVideoExample);
        if (CollectionUtils.isEmpty(matchVideos)) {
            return null;
        }
        return matchVideos.get(0);
    }

    public void updateMatchContent(PublishContent publishContent,
                                   List<PublishMiniprogram> publishMiniprogramList, MatchVideo matchVideo) {
        try {
            JSONArray jsonArray = getResponse(publishMiniprogramList);
            String traceId = matchVideo.getTraceId();
            MatchVideo updateMatchVideo = new MatchVideo();
            updateMatchVideo.setId(matchVideo.getId());
            updateMatchVideo.setContentStatus(4);
            updateMatchVideo.setPublishFlag(2);
            updateMatchVideo.setResponse(jsonArray.toJSONString());
            matchVideoMapper.updateByPrimaryKeySelective(updateMatchVideo);
            publicContentService.updatePublishContentTraceId(traceId, publishContent.getId());
        } catch (Exception e) {
            log.error("updateMatchContent error", e);
        }
    }

    private JSONArray getResponse(List<PublishMiniprogram> publishMiniprogramList) {
        JSONArray jsonArray = new JSONArray();
        for (PublishMiniprogram publishMiniprogram : publishMiniprogramList) {
            //生成match表json数据
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("uid", publishMiniprogram.getUserId());
            jsonObject.put("source", publishMiniprogram.getSource());
            jsonObject.put("kimiTitle", publishMiniprogram.getVideoTitle());
            jsonObject.put("videoId", publishMiniprogram.getVideoId());
            jsonObject.put("videoCover", publishMiniprogram.getVideoCover());
            jsonObject.put("videoPath", publishMiniprogram.getVideoPath());
            jsonObject.put("videoOss", publishMiniprogram.getVideoOssPath());
            jsonObject.put("rootSourceId", publishMiniprogram.getRootSourceId());
            jsonArray.add(jsonObject);
        }
        return jsonArray;
    }

    public List<PublishArticleData> getWaitingSort(PlanAccount planAccount) {
        //获取待排序数据
        GetContentsParam waitingSortParam = new GetContentsParam();
        waitingSortParam.setPlanId(planAccount.getPlanId());
        waitingSortParam.setAccountId(planAccount.getAccountId());
        waitingSortParam.setPageNum(1);
        waitingSortParam.setPageSize(planAccount.getPublishPreNum());
        LongArticleSystemContentVO longArticleSystemContentVO = aigcService.listWaitingSortContents(waitingSortParam);
        if (longArticleSystemContentVO == null
                || longArticleSystemContentVO.getTotalCount() < planAccount.getPublishPreMinNum()
                || CollectionUtils.isEmpty(longArticleSystemContentVO.getContentItemList())
                || longArticleSystemContentVO.getContentItemList().size() < planAccount.getPublishPreMinNum()) {
            return new ArrayList<>();
        }
        log.info("aigcService.listWaitingSortContents.total={}", longArticleSystemContentVO.getTotalCount());
        List<PublishArticleData> waitSortList = new ArrayList<>();
        for (ContentItemVO contentItemVO : longArticleSystemContentVO.getContentItemList()) {
            PublishArticleData publishArticleData = new PublishArticleData();
            publishArticleData.setId(contentItemVO.getPublishContentId());
            publishArticleData.setSourceId(contentItemVO.getSourceId());
            publishArticleData.setTitle(contentItemVO.getTitle());
            publishArticleData.setContentPoolType(contentItemVO.getFlowPoolLevelTag());
            publishArticleData.setSourceType(contentItemVO.getSourceType());
            waitSortList.add(publishArticleData);
        }
        return waitSortList;
    }

    public List<PublishArticleData> getSortList(PlanAccount planAccount) {
        ArticleSortRequest articleSortRequest = new ArticleSortRequest();
        articleSortRequest.setAccountId(planAccount.getAccountId());
        articleSortRequest.setAccountName(planAccount.getAccountName());
        articleSortRequest.setGhId(planAccount.getGhId());
        //测试参数
//            articleSortRequest.setStrategy("ArticleRankRandom");
//            articleSortRequest.setExcludeLog(true);
        articleSortRequest.setStrategy(planAccount.getSortStrategy());
        articleSortRequest.setPublishNum(planAccount.getPublishPreNum());
        if (Objects.equals(PublishGzhPushTypeEnum.auto_group_publish.getVal(), planAccount.getPushType())) {
            articleSortRequest.setPublishNum(planAccount.getPublishNum());
        }
        articleSortRequest.setPlanId(planAccount.getPlanId());
        articleSortRequest.setPushType(planAccount.getPushType());
        if (StringUtils.isNotEmpty(planAccount.getExcludeContentIndex())) {
            List<Integer> excludeContentIndexList = JSONArray.parseArray(planAccount.getExcludeContentIndex(), Integer.class);
            if (!CollectionUtils.isEmpty(excludeContentIndexList)) {
                articleSortRequest.setExcludeContentIndex(excludeContentIndexList);
            }
        }
        log.info("articleSortRequest={}", JSONObject.toJSONString(articleSortRequest));
        ArticleSortResponse articleSortResponse = sortService.publishArticleSort(articleSortRequest);

        if (articleSortResponse == null || articleSortResponse.getData() == null) {
            log.info("articleSortResponse={}", articleSortResponse);
            String message = String.format("排序结果为空 planId=%s accountName=%s", planAccount.getPlanId(), planAccount.getAccountName());
            LarkRobotUtil.sendMessage(message);

            return new ArrayList<>();
        }
        ArticleSortResponseData data = articleSortResponse.getData();
        if (!CollectionUtils.isEmpty(data.getFilter_list())) {
            List<FilterSortContentParam> list = new ArrayList<>();
            for (PublishArticleData publishArticleData : data.getFilter_list()) {
                FilterSortContentParam param = new FilterSortContentParam();
                param.setFilterReason(publishArticleData.getFilterReason());
                param.setPublishContentId(publishArticleData.getId());
                list.add(param);
            }
            FilterSortParam param = new FilterSortParam();
            param.setFilterSortContentList(list);
            aigcService.filterSortContents(param);
        }
        if (CollectionUtils.isEmpty(articleSortResponse.getData().getRank_list())
                || articleSortResponse.getData().getRank_list().size() < planAccount.getPublishPreMinNum()) {
            log.info("rank_list = {}", articleSortResponse.getData().getRank_list());
            String message = String.format("排序结果不足 planId=%s accountName=%s num=%s",
                    planAccount.getPlanId(), planAccount.getAccountName(), articleSortResponse.getData().getRank_list().size());
            LarkRobotUtil.sendMessage(message);
            return new ArrayList<>();
        }
        return data.getRank_list();
    }

    public List<VideoDetail> getPublishVideoDetail(PublishContent publishContent, PlanAccount planAccount, MatchVideo matchVideo) {
        List<CrawlerVideo> contentMiniVideos = getContentMiniVideo(publishContent, planAccount);
        log.info("getPublishVideoDetail contentMiniVideos={}", contentMiniVideos);
        if (CollectionUtils.isEmpty(contentMiniVideos)) {
            LarkRobotUtil.sendMessage("文章获取小程序为空 publishContent.getSourceId=" + publishContent.getSourceId());
            return new ArrayList<>();
        }
        List<VideoDetail> videoDetails = publishVideo(contentMiniVideos, matchVideo);
        log.info("getPublishVideoDetail videoDetails={}", videoDetails);
        if (CollectionUtils.isEmpty(videoDetails)) {
            LarkRobotUtil.sendMessage("发布后小程序为空 publishContent.getSourceId=" + publishContent.getSourceId());
            return new ArrayList<>();
        }
        return videoDetails;
    }

    public List<VideoDetail> getOnlyMiniPublishVideoDetail(PublishContent publishContent) {
        List<VideoDetail> res = new ArrayList<>();
        SingleVideoSourceExample example = new SingleVideoSourceExample();
        example.createCriteria().andContentTraceIdEqualTo(publishContent.getSourceId()).andAuditStatusEqualTo(1);
        List<SingleVideoSource> singleVideoSources = singleVideoSourceMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(singleVideoSources)) {
            return null;
        }
        SingleVideoSource singleVideoSource = singleVideoSources.get(0);
        VideoDetail videoDetail = videoService.publish(singleVideoSource.getVideoOssPath(),
                SINGLE_VIDEO_UID, singleVideoSource.getMiniProgramTitle());
        videoDetail.setCrawlerVideoId(singleVideoSource.getId());
        videoDetail.setKimiTitle(singleVideoSource.getMiniProgramTitle());
        videoDetail.setUid(SINGLE_VIDEO_UID);
        videoDetail.setVideoOss(singleVideoSource.getVideoOssPath());
        String traceId = "direct-" + UUID.randomUUID() + "-" + System.currentTimeMillis() / 1000;
        videoDetail.setTraceId(traceId);
        res.add(videoDetail);
        return res;
    }

    public List<VideoDetail> publishVideo(List<CrawlerVideo> contentMiniVideos, MatchVideo matchVideo) {
        List<VideoDetail> videoDetails = new ArrayList<>();
        //发布小程序
        for (CrawlerVideo crawlerVideo : contentMiniVideos) {
            Integer crawlerVideoId = crawlerVideo.getId();
            String videoOssPath = crawlerVideo.getVideoOssPath();
            String platform = crawlerVideo.getPlatform();
            String userId = crawlerVideo.getUserId();
            String traceId = matchVideo.getTraceId();
            LongArticlesText kimiText = kimiService.getKimiText(crawlerVideo.getContentId());
            if (kimiText == null || StringUtils.isEmpty(kimiText.getKimiTitle())) {
                log.error("crawlerVideo kimiText is null contentId={}", crawlerVideo.getContentId());
                continue;
            }
            String kimiTitle = kimiText.getKimiTitle();
            VideoDetail videoDetail = videoService.publish(videoOssPath, userId, kimiTitle);
            if (videoDetail == null) {
                continue;
            }
            videoDetail.setCrawlerVideoId(crawlerVideoId);
            videoDetail.setKimiTitle(kimiTitle);
            videoDetail.setUid(userId);
            videoDetail.setVideoOss(videoOssPath);
            videoDetail.setSource(platform);
            videoDetail.setTraceId(traceId);
            videoDetails.add(videoDetail);
        }
        return videoDetails;
    }


    public List<CrawlerVideo> getContentMiniVideo(PublishContent publishContent, PlanAccount planAccount) {
        try {
            CrawlerVideoExample example = new CrawlerVideoExample();
            example.setOrderByClause("score desc");
            example.createCriteria().andDownloadStatusEqualTo(2)
                    .andContentIdEqualTo(publishContent.getSourceId())
                    .andStatusEqualTo(1)
                    .andIsIllegalEqualTo(0);
            List<CrawlerVideo> crawlerVideoList = crawlerVideoMapper.selectByExample(example);
            List<CrawlerVideo> videoList = new ArrayList<>();
            if (!CollectionUtils.isEmpty(crawlerVideoList)) {
                videoList.add(crawlerVideoList.get(0));
            }
            log.info("getContentMiniVideo videoList={}", videoList);
            return videoList;
        } catch (Exception e) {
            log.error("getContentMiniVideo error", e);
        }
        return new ArrayList<>();
    }

}
