package com.tzld.piaoquan.longarticle.service.local.impl;

import com.tzld.piaoquan.longarticle.dao.mapper.PlanAccountMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.PlanMapper;
import com.tzld.piaoquan.longarticle.model.po.Plan;
import com.tzld.piaoquan.longarticle.model.po.PlanAccount;
import com.tzld.piaoquan.longarticle.model.po.PlanAccountExample;
import com.tzld.piaoquan.longarticle.model.po.PlanExample;
import com.tzld.piaoquan.longarticle.service.local.PlanAccountService;
import com.tzld.piaoquan.longarticle.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

@Service
public class PlanAccountServiceImpl implements PlanAccountService {

    @Autowired
    private PlanAccountMapper planAccountMapper;

    @Autowired
    private PlanMapper planMapper;

    public void saveOrUpdatePlanAccount(PlanAccount planAccount) {
        PlanAccountExample example = new PlanAccountExample();
        example.createCriteria().andAccountIdEqualTo(planAccount.getAccountId())
                .andPlanIdEqualTo(planAccount.getPlanId())
                .andCreateTimeGreaterThan(DateUtil.getThatDayDate());
        List<PlanAccount> planAccounts = planAccountMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(planAccounts)) {
            planAccountMapper.insertSelective(planAccount);
        } else {
            planAccount.setId(planAccounts.get(0).getId());
            planAccount.setIsDelete(0);
            planAccountMapper.updateByPrimaryKeySelective(planAccount);
        }
    }

    @Override
    public void saveOrUpdatePlan(Plan plan) {
        PlanExample example = new PlanExample();
        example.createCriteria().andPlanIdEqualTo(plan.getPlanId());
        List<Plan> plans = planMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(plans)) {
            planMapper.insertSelective(plan);
        } else {
            plan.setId(plans.get(0).getId());
            plan.setIsDelete(0);
            planMapper.updateByPrimaryKeySelective(plan);
        }
    }

    @Override
    public List<PlanAccount> getPlanAccountByPlanId(String planId) {
        PlanAccountExample example = new PlanAccountExample();
        example.createCriteria().andPlanIdEqualTo(planId)
                .andCreateTimeGreaterThan(DateUtil.getThatDayDate());
        return planAccountMapper.selectByExample(example);
    }

    public List<PlanAccount> getMatchPlanAccount() {
        PlanAccountExample example = new PlanAccountExample();
        example.createCriteria().andCreateTimeGreaterThan(DateUtil.getThatDayDate())
                .andMatchStatusEqualTo(0).andIsDeleteEqualTo(0);
        return planAccountMapper.selectByExample(example);
    }

    public List<PlanAccount> getNormalPlanAccount() {
        PlanAccountExample example = new PlanAccountExample();
        example.createCriteria().andCreateTimeGreaterThan(DateUtil.getThatDayDate())
                .andStatusEqualTo(0).andIsDeleteEqualTo(0).andPublishOpenFlagEqualTo(1);
        return planAccountMapper.selectByExample(example);
    }

    public List<PlanAccount> getPlanAccount(String planId, String accountId) {
        PlanAccountExample example = new PlanAccountExample();
        example.createCriteria().andCreateTimeGreaterThan(DateUtil.getThatDayDate())
                .andPlanIdEqualTo(planId)
                .andAccountIdEqualTo(accountId);
        return planAccountMapper.selectByExample(example);
    }

    public List<Plan> getPlanList() {
        PlanExample example = new PlanExample();
        List<Plan> plans = planMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(plans)) {
            return new ArrayList<>();
        }
        return plans;
    }

    public void delPlan(String planId) {
        PlanExample planExample = new PlanExample();
        planExample.createCriteria().andPlanIdEqualTo(planId);
        Plan updatePlan = new Plan();
        updatePlan.setIsDelete(1);
        planMapper.updateByExampleSelective(updatePlan, planExample);

        PlanAccountExample planAccountExample = new PlanAccountExample();
        planAccountExample.createCriteria().andPlanIdEqualTo(planId).andCreateTimeGreaterThan(DateUtil.getThatDayDate());
        PlanAccount updatePlanAccount = new PlanAccount();
        updatePlanAccount.setIsDelete(1);
        planAccountMapper.updateByExampleSelective(updatePlanAccount, planAccountExample);
    }

    @Override
    public void delAccount(String planId, String accountId) {
        PlanAccountExample planAccountExample = new PlanAccountExample();
        planAccountExample.createCriteria()
                .andAccountIdEqualTo(accountId)
                .andPlanIdEqualTo(planId).
                andCreateTimeGreaterThan(DateUtil.getThatDayDate());
        PlanAccount updatePlanAccount = new PlanAccount();
        updatePlanAccount.setIsDelete(1);
        planAccountMapper.updateByExampleSelective(updatePlanAccount, planAccountExample);
    }


    @Override
    public void updateMatchStatus(Integer status, Long id) {
        PlanAccount updatePlanAccount = new PlanAccount();
        updatePlanAccount.setId(id);
        updatePlanAccount.setMatchStatus(status);
        planAccountMapper.updateByPrimaryKeySelective(updatePlanAccount);
    }

    @Override
    public void updateStatus(Integer status, Long id) {
        PlanAccount updatePlanAccount = new PlanAccount();
        updatePlanAccount.setId(id);
        updatePlanAccount.setStatus(status);
        planAccountMapper.updateByPrimaryKeySelective(updatePlanAccount);
    }

    public PlanAccount getPlanAccountById(Long id) {
        return planAccountMapper.selectByPrimaryKey(id);
    }

    public void updateRetry(PlanAccount planAccount) {
        PlanAccount updatePlanAccount = new PlanAccount();
        updatePlanAccount.setId(planAccount.getId());
        updatePlanAccount.setRetryCount(planAccount.getRetryCount() + 1);
        planAccountMapper.updateByPrimaryKeySelective(updatePlanAccount);
    }

    public List<PlanAccount> getPlanAccount(String accountId) {
        PlanAccountExample example = new PlanAccountExample();
        example.createCriteria().andCreateTimeGreaterThan(DateUtil.getThatDayDate())
                .andAccountIdEqualTo(accountId);
        return planAccountMapper.selectByExample(example);
    }


}
