package com.tzld.piaoquan.longarticle.utils;

import lombok.Getter;

import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;

public class TimeZoneUtil {

    @Getter
    public enum Timezone {
        china("Asia/Shanghai", "中国"),
        america("America/New_York", "美国");
        private final String val;
        private final String description;

        Timezone(String val, String description) {
            this.val = val;
            this.description = description;
        }
    }

    private TimeZoneUtil() {
    }

    public static int toExcelSerialNumber(String dateStr) {
        // 定义日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        // 解析输入日期
        LocalDate date = LocalDate.parse(dateStr, formatter);
        // Excel的基准日期
        LocalDate baseDate = LocalDate.of(1900, 1, 1);
        // 计算天数差
        long daysBetween = ChronoUnit.DAYS.between(baseDate, date);
        // Excel将1900年2月29日（不存在的日期）算作有效日期，因此需要加1
        return (int) daysBetween + 2;
    }

    public static LocalTime currentTime(Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        return LocalTime.now(zoneId);
    }


    public static long todayMinTimestamp(Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate currentDate = LocalDate.now(zoneId);
        LocalDateTime maxDateTime = LocalDateTime.of(currentDate, LocalTime.MIN);
        return maxDateTime.atZone(zoneId).toInstant().toEpochMilli();
    }

    public static long dayBeforeTimestamp(Timezone timezone, int dayNum) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDateTime now = LocalDateTime.now(zoneId);
        return now.minusDays(dayNum).atZone(zoneId).toInstant().toEpochMilli();
    }

    public static LocalDateTime parseDate(String dateStr, DateTimeFormatter dateTimeFormatter) {
        return LocalDateTime.parse(dateStr, dateTimeFormatter);
    }

    public static long dateToTimestamp(Timezone timezone, LocalDateTime localDateTime) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        return localDateTime.atZone(zoneId).toInstant().toEpochMilli();
    }

    public static long hourBeforeTimestamp(Timezone timezone, int hourNUm) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDateTime now = LocalDateTime.now(zoneId);
        return now.minusHours(hourNUm).atZone(zoneId).toInstant().toEpochMilli();
    }

    public static String todayYMD(Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate today = LocalDate.now(zoneId);
        return today.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
    }

    public static String getDateStr(Timezone timezone, int days) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate day = LocalDate.now(zoneId).plusDays(days);
        return day.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
    }

    public static String nowTime(Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalTime now = LocalTime.now(zoneId);
        return now.format(DateTimeFormatter.ofPattern("HH:mm:ss"));
    }

    public static String yesterdayYMD(Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate yesterday = LocalDate.now(zoneId).minusDays(1);
        return yesterday.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
    }

    public static String beforeDayYMD(Timezone timezone, Integer dayNum) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate yesterday = LocalDate.now(zoneId).minusDays(dayNum);
        return yesterday.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
    }

    public static String beforeDay(Timezone timezone, Integer dayNum, DateTimeFormatter dateTimeFormatter) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate yesterday = LocalDate.now(zoneId).minusDays(dayNum);
        return yesterday.format(dateTimeFormatter);
    }

    public static String toDay(Long epochMilli, Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        Instant instant = Instant.ofEpochMilli(epochMilli);
        return LocalDateTime.ofInstant(instant, zoneId).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
    }

    public static String toDayTime(Long epochMilli, Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        Instant instant = Instant.ofEpochMilli(epochMilli);
        return LocalDateTime.ofInstant(instant, zoneId).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }


    public static long todayMaxTimestamp(Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate currentDate = LocalDate.now(zoneId);
        LocalDateTime maxDateTime = LocalDateTime.of(currentDate, LocalTime.MAX);
        return maxDateTime.atZone(zoneId).toInstant().toEpochMilli();
    }

    public static long getDateMinTimestamp(String dateStr, Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate localdate = LocalDate.parse(dateStr);
        LocalDateTime dateTime = LocalDateTime.of(localdate, LocalTime.MIN);
        return dateTime.atZone(zoneId).toInstant().toEpochMilli();
    }

    public static long getDateMaxTimestamp(String dateStr, Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate localdate = LocalDate.parse(dateStr);
        LocalDateTime dateTime = LocalDateTime.of(localdate, LocalTime.MAX);
        return dateTime.atZone(zoneId).toInstant().toEpochMilli();
    }

    public static int getTodayDayOfWeek(Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        LocalDate currentDate = LocalDate.now(zoneId);
        return currentDate.getDayOfWeek().getValue();
    }

    public static String format(Timezone timezone, Long epochMilli, DateTimeFormatter dateTimeFormatter) {
        return Instant.ofEpochMilli(epochMilli).atZone(ZoneId.of(timezone.val)).format(dateTimeFormatter);
    }

    public static Long getNearestHalfHourTimestamp(Timezone timezone) {
        LocalDateTime now = LocalDateTime.now(ZoneId.of(timezone.val));
        LocalDateTime nearestHalfHour = now.withMinute(now.getMinute() > 30 ? 30 : 0).withSecond(0);
        return nearestHalfHour.atZone(ZoneId.of(timezone.val)).toEpochSecond() * 1000;
    }

    public static Integer getDateInterval(String dateStr, Timezone timezone) {
        Long start = getDateMinTimestamp(dateStr, timezone);
        Long end = todayMinTimestamp(timezone);
        return (int) Math.floor((end - start) / (24 * 3600 * 1000));
    }

    public static String toYMD(Long timestamp, Timezone timezone) {
        return Instant.ofEpochMilli(timestamp).atZone(ZoneId.of(timezone.val)).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
    }

    public static String toYMDT(Long timestamp) {

        // 将时间戳转换为Instant对象
        Instant instant = Instant.ofEpochMilli(timestamp);

        // 将Instant对象转换为ZonedDateTime对象，指定时区为UTC
        ZonedDateTime zdt = instant.atZone(ZoneOffset.UTC);

        // 定义ISO 8601格式的DateTimeFormatter
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");

        // 将ZonedDateTime对象格式化为字符串
        return zdt.format(formatter);
    }

    public static long getDateStrTimestamp(String dateTimeString, Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        // 定义日期时间格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");

        // 解析字符串为 LocalDateTime 对象
        LocalDateTime localDateTime = LocalDateTime.parse(dateTimeString, formatter);

        // 将 LocalDateTime 转换为时间戳
        return localDateTime.atZone(zoneId).toInstant().toEpochMilli();
    }

    public static long getDateStrTimestamp(String dateTimeString, String pattern, Timezone timezone) {
        ZoneId zoneId = ZoneId.of(timezone.val);
        // 定义日期时间格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(pattern);

        // 解析字符串为 LocalDateTime 对象
        LocalDateTime localDateTime = LocalDateTime.parse(dateTimeString, formatter);

        // 将 LocalDateTime 转换为时间戳
        return localDateTime.atZone(zoneId).toInstant().toEpochMilli();
    }

    public static List<String> getBeforeDaysStr(String dateStr, int beforeDayNum) {
        List<String> result = new ArrayList<>();
        LocalDate localDate = LocalDate.parse(dateStr, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        for (int i = 0; i < beforeDayNum; i++) {
            result.add(localDate.minusDays(i + 1).format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        }
        return result;
    }
}